/**
    example04a.java  Utility that will write a properties list to a SmartDongle.
                    The properties list will be read by an example application
                    program, example04b.java



    The map of the data stored to SmartDongle:

           16 bytes              2 bytes                  n bytes
    | Customer's AES key | Property Object size |    Customer's Property    |
     -------------------   --------------------  ---------------------------
     0     field 1     15 16      field 2     17 18         field 3         n


    MicroWorks, Inc.
    2808 North Cole Road
    Boise, ID 83704

    www.smartdongle.com


    ***** BEGIN LICENSE BLOCK *****

    Version: MPL 1.1/GPL 2.0/LGPL 2.1

    The contents of this file are subject to the Mozilla Public License
    Version 1.1 (the "License"); you may not use this file except in
    compliance with the License. You may obtain a copy of the License at
    http://www.mozilla.org/MPL/

    Software distributed under the License is distributed on an "AS IS" basis,
    WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
    for the specific language governing rights and limitations under the
    License.

    The Original Code is MicroWorks, Inc. code.

    The Initial Developer of the Original Code is MicroWorks, Inc. Portions
    created by MicroWorks, Inc. are Copyright (C) 2010
    MicroWorks, Inc. All Rights Reserved.

    Contributor(s):

    Alternatively, the contents of this file may be used under the terms of
    either the GNU General Public License Version 2 or later (the "GPL"), or
    the GNU Lesser General Public License Version 2.1 or later (the "LGPL"),
    in which case the provisions of the GPL or the LGPL are applicable instead
    of those above. If you wish to allow use of your version of this file only
    under the terms of either the GPL or the LGPL, and not to allow others to
    use your version of this file under the terms of the MPL, indicate your
    decision by deleting the provisions above and replace them with the notice
    and other provisions required by the GPL or the LGPL. If you do not delete
    the provisions above, a recipient may use your version of this file under
    the terms of any one of the MPL, the GPL or the LGPL.

    ***** END LICENSE BLOCK *****
*/

package com.microworks;

import java.lang.reflect.Array;
import java.io.ByteArrayOutputStream;
import java.util.Properties;
import javax.crypto.Cipher;
import javax.crypto.spec.SecretKeySpec;
import com.microworks.SmartDongle;

/*
    The map of the data stored to SmartDongle:

           16 bytes              2 bytes                  n bytes
    | Customer's AES key | Property Object size |    Customer's Property    |
     -------------------   --------------------  ---------------------------
     0     field 1     15 16      field 2     17 18         field 3         n

*/

public class Example4a {
  private static int error;

  // Your keys here
  private final static long p1 = 0xEC6CC589AEFD1E75L;
  private final static long p2 = 0xFCEC0A6A82747B3FL;

  /*
    This is a demo 128 bit encryption key,
    chose some another 16 character key for your project.
  */
  private final static byte[] AESkey = {
    '7',  'o',  'b',  'd',
    'g',  'E',  'n',  '2',
    '\'', 'T',  'H',  '>',
    ']',  'S',  'E',  '|'};


  // Parameters needed to store Properties object to SmartDongle.
  private final static int aesBlockSize = 128;
  private final static int keyAddress = 0;
  private final static int propertiesSizeAddress = aesBlockSize / 8;
  private final static int propertiesObjectAddress = propertiesSizeAddress + 2;

  public static void main(String args[]) {
    /**
     * Initialise your Properties object that will be saved to SmartDongle.
     */
    Properties properties = new Properties();
    properties.setProperty("customerName", "Widget City, Inc.");
    properties.setProperty("licenseSN", "1111-2222-3333-4444-5555-6666");
    properties.setProperty("licenseExpiration", "30 May 2010");
    properties.setProperty("usageCurrent", "0");
    properties.setProperty("usageLimit", "0");

    // Properties comment.
    String propertiesHeader = "Example property list stored on a SmartDongle";

    /*
        This code will process and save your Properties to a SmartDongle
        1) Instantiate an AES key from the user's key bytes.
        2) Initialize an AES cipher for encryption.
        3) Write the user's Properties into an output byte stream.
        4) Encrypt user's Properties byte stream.
        5) Save user's raw key bytes to SmartDongle
        6) Save length of user's encrypted Properties byte stream to Smartdongle
        7) Save encrypted Properties byte stream to SmartDongle.
    */
    int cipherTextLength = 0;
    try {
      int i;

      /*
        Instantiate an AES key from the user's key bytes.
        Initialize an AES cipher for encryption.
      */
      SecretKeySpec skeySpec = new SecretKeySpec(AESkey, "AES");
      Cipher cipher = Cipher.getInstance("AES");
      cipher.init(Cipher.ENCRYPT_MODE, skeySpec);

      /*
        Write the user's Properties into an output byte stream.
        Encrypt user's Properties byte stream.
      */
      ByteArrayOutputStream baos = new ByteArrayOutputStream();
      properties.store(baos, propertiesHeader);

      // Convert output byte stream into a byte array
      byte[] propertiesByteArray = baos.toByteArray();

      // Encrypt the converted properties list
      byte[] cipherText = cipher.doFinal(propertiesByteArray);
      cipherTextLength = Array.getLength(cipherText);

      /*
        Write user's raw key bytes to SmartDongle
      */
      error = SmartDongle.Write(p1,
                               p2,
                               keyAddress,
                               AESkey,
                               aesBlockSize / 8);  // block size is in bits
      if (error != 0) {
        System.out.println("Error writing user's key bytes: " + error);
        return;
      }

      // Verify that key was written
      byte[] field1 = new byte[aesBlockSize / 8];
      error = SmartDongle.Read(p1,
                              p2,
                              keyAddress,
                              field1,
                              aesBlockSize / 8);  // block size is in bits
      for (i = 0; i < aesBlockSize / 8; i++) {
        if (AESkey[i] != field1[i]) {
          System.out.println("Failed to write user's key bytes: mismatch");
          return;
        }
      }

      System.out.println("Success writing AES key of length " +
                         Array.getLength(AESkey));

      /*
        Save length of user's encrypted Properties byte stream to Smartdongle.
      */

      // We need to record the properties list cipher length
      Short s0 = new Short((short)(cipherTextLength & 0xff));
      Short s1 = new Short((short)((cipherTextLength & 0xff00) >> 8));

      // Convert the shorts above into a byte array
      byte[] propertiesSize = new byte[2];
      propertiesSize[0] = s0.byteValue();
      propertiesSize[1] = s1.byteValue();

      // Write out the byte array with length of properties list
      error = SmartDongle.Write(p1,
                               p2,
                               propertiesSizeAddress,
                               propertiesSize,
                               2);
      if (error != 0) {
        System.out.println("Error writing properties length field: " +
                           error);
        return;
      }

      // Verify that properties list length was written
      byte[] field2 = new byte[2];
      error = SmartDongle.Read(p1,
                              p2,
                              propertiesSizeAddress,
                              field2,
                              2);

      for (i = 0; i < 2; i++) {
        if (propertiesSize[i] != field2[i]) {
          System.out.println("Failed to write length field: mismatch");
          return;
        }
      }

      System.out.println("Success writing properties list length field ");

      /*
        Save encrypted Properties byte stream to SmartDongle.
      */
      error = SmartDongle.Write(p1,
                               p2,
                               propertiesObjectAddress,
                               cipherText,
                               cipherTextLength);
      if (error != 0) {
        System.out.println("Error writing properties field: " +
                           error);
        return;
      }

      // Verify that properties list was written
      byte[] field3 = new byte[cipherTextLength];
      error = SmartDongle.Read(p1,
                              p2,
                              propertiesObjectAddress,
                              field3,
                              cipherTextLength);
      for (i = 0; i < cipherTextLength; i++) {
        if (cipherText[i] != field3[i]) {
          System.out.println("Failed to write properties field: mismatch");
          return;
        }
      }
    }
    catch (java.io.IOException ioe) {
      System.out.println("Caught Exception:  " + ioe);
    }
    catch (java.security.NoSuchAlgorithmException nsafe) {
      System.out.println("Caught Exception:  " + nsafe);
    }
    catch (javax.crypto.NoSuchPaddingException nspe) {
      System.out.println("Caught Exception:  " + nspe);
    }
    catch (javax.crypto.BadPaddingException bpe) {
      System.out.println("Caught Exception:  " + bpe);
    }
    catch (javax.crypto.IllegalBlockSizeException ibse) {
      System.out.println("Caught Exception:  " + ibse);
    }
    catch (java.security.InvalidKeyException ike) {
      System.out.println("Caught Exception:  " + ike);
    }

    // Get serial number for report
    StringBuffer smartDongleSerialNumber = new StringBuffer();
    SmartDongle.GetSerialNumber(smartDongleSerialNumber);

    System.out.println("Finished writing property list of length "
                       + cipherTextLength
                       + " to sn:"
                       + smartDongleSerialNumber.toString());
  }
}
