/**
    example04b.java

    Retrieve a user's properties object written with example04a.java utility.


    The map of the data stored to SmartDongle:

           16 bytes              2 bytes                  n bytes
    | Customer's AES key | Property Object size |    Customer's Property    |
     -------------------   --------------------  ---------------------------
     0     field 1     15 16      field 2     17 18         field 3         n


    MicroWorks, Inc.
    2808 North Cole Road
    Boise, ID 83704

    www.smartdongle.com


    ***** BEGIN LICENSE BLOCK *****

    Version: MPL 1.1/GPL 2.0/LGPL 2.1

    The contents of this file are subject to the Mozilla Public License
    Version 1.1 (the "License"); you may not use this file except in
    compliance with the License. You may obtain a copy of the License at
    http://www.mozilla.org/MPL/

    Software distributed under the License is distributed on an "AS IS" basis,
    WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
    for the specific language governing rights and limitations under the
    License.

    The Original Code is MicroWorks, Inc. code.

    The Initial Developer of the Original Code is MicroWorks, Inc. Portions
    created by MicroWorks, Inc. are Copyright (C) 2010
    MicroWorks, Inc. All Rights Reserved.

    Contributor(s):

    Alternatively, the contents of this file may be used under the terms of
    either the GNU General Public License Version 2 or later (the "GPL"), or
    the GNU Lesser General Public License Version 2.1 or later (the "LGPL"),
    in which case the provisions of the GPL or the LGPL are applicable instead
    of those above. If you wish to allow use of your version of this file only
    under the terms of either the GPL or the LGPL, and not to allow others to
    use your version of this file under the terms of the MPL, indicate your
    decision by deleting the provisions above and replace them with the notice
    and other provisions required by the GPL or the LGPL. If you do not delete
    the provisions above, a recipient may use your version of this file under
    the terms of any one of the MPL, the GPL or the LGPL.

    ***** END LICENSE BLOCK *****
*/
package com.microworks;

import java.io.ByteArrayInputStream;
import java.util.Properties;
import javax.crypto.Cipher;
import javax.crypto.spec.SecretKeySpec;
import com.microworks.SmartDongle;

/*
    The map of the data stored to SmartDongle:

           16 bytes              2 bytes                  n bytes
    | Customer's AES key | Property Object size |    Customer's Property    |
     -------------------   --------------------  ---------------------------
     0     field 1     15 16      field 2     17 18         field 3         n

*/

public class Example4b {

  private static int error;

  // Your keys here
  private final static long p1 = 0xEC6CC589AEFD1E75L;
  private final static long p2 = 0xFCEC0A6A82747B3FL;

  // Your properties to be retrieved from a SmartDongle
  private static Properties properties;

  // Parameters needed to retrieve Properties object from SmartDongle.
  private final static int aesBlockSize = 128;
  private final static int keyAddress = 0;
  private final static int propertiesSizeAddress = aesBlockSize / 8;
  private final static int propertiesObjectAddress = propertiesSizeAddress + 2;

  public static void main(String args[]) {
    /*
        This code will retrieve your Properties object from a SmartDongle

        1) Read user's raw key bytes from Smartdongle.
        2) Instantiate an AES key from the user's key bytes.
        3) Initialize an AES cipher for encryption.
        4) Read the encrypted input stream length from Smartdongle.
        5) Read the encrypted input stream from SmartDongle.
        6) Decrypt the input stream.
        7) Load the user's Properties from the decrypted input stream.
    */
    int cipherTextLength = 0;
    try {
      /*
        1) Read user's raw key bytes from Smartdongle.
      */
      byte[] field1 = new byte[aesBlockSize / 8];
      error = SmartDongle.Read(p1,
                              p2,
                              keyAddress,
                              field1,
                              aesBlockSize / 8);  // block size is in bits
      if (error != 0) {
        // FAIL exiting...
        return;
      }

      /*
        2) Instantiate an AES key from the user's key bytes.
        3) Initialize an AES cipher for encryption.
      */
      SecretKeySpec skeySpec = new SecretKeySpec(field1, "AES");
      Cipher cipher = Cipher.getInstance("AES");
      cipher.init(Cipher.DECRYPT_MODE, skeySpec);

      /*
        4) Read the encrypted input stream length from Smartdongle.
      */
      byte[] field2 = new byte[2];
      error = SmartDongle.Read(p1,
                              p2,
                              propertiesSizeAddress,
                              field2,
                              2);
      if (error != SmartDongle.USK_OK) {
        // FAIL exiting...
        return;
      }

      /*
        The byte array length was written as unsigned data,
        and the serialised properties length must be <= 32767. (user memory)
      */
      int field3Length =
        (field2[1] >= 0 ? field2[1] : 256 + field2[1]) << 8;
      field3Length += field2[0] >= 0 ? field2[0] : 256 + field2[0];

      /*
        5) Read the encrypted input stream from SmartDongle.
      */
      byte[] field3 = new byte[field3Length];
      error = SmartDongle.Read(p1,
                              p2,
                              propertiesObjectAddress,
                              field3,
                              field3Length);
      if (error != SmartDongle.USK_OK) {
        // FAIL exiting...
        return;
      }

      /*
        6) Decrypt the input stream.
      */
      byte[] decipheredStream = cipher.doFinal(field3);

      /*
        7) Load the user's Properties from the decrypted input stream.
      */
      properties = new Properties();
      ByteArrayInputStream bais = new ByteArrayInputStream(decipheredStream);
      properties.load(bais);
    }
    catch (java.io.IOException ioe) {
      System.out.println("Caught Exception:  " + ioe);
    }
    catch (java.security.NoSuchAlgorithmException nsafe) {
      System.out.println("Caught Exception:  " + nsafe);
    }
    catch (javax.crypto.NoSuchPaddingException nspe) {
      System.out.println("Caught Exception:  " + nspe);
    }
    catch (javax.crypto.BadPaddingException bpe) {
      System.out.println("Caught Exception:  " + bpe);
    }
    catch (javax.crypto.IllegalBlockSizeException ibse) {
      System.out.println("Caught Exception:  " + ibse);
    }
    catch (java.security.InvalidKeyException ike) {
      System.out.println("Caught Exception:  " + ike);
    }

    /*
       Your main class code goes here
    */

    System.out.println("Success: Your application can use these properties:");

    System.out.println("    Customer Name: " +
                       properties.getProperty("customerName"));

    System.out.println("    Serial Number: " +
                       properties.getProperty("licenseSN"));

    System.out.println("    Expiration Date: " +
                       properties.getProperty("licenseExpiration"));

    System.out.println("    Usage: " +
                       properties.getProperty("usageCurrent"));

    System.out.println("    Usage Limit: " +
                       properties.getProperty("usageLimit"));
  }
}
